#ifndef ROUT_H
#define ROUT_H

/* Timer perdiod in milliseconds */
#define PERIOD 1000

/* Number of columns in topography */
#define COLUMNS 4

/* Message type identifiers */
#define TYPE_ANNOUNCEMENT 11
#define TYPE_PATH 13
#define TYPE_NODE_CHANGE 14
#define TYPE_JOIN2 15	

/* Maximum quadratic distance radio can reach */
//#define MAXDISTANCE 5
#define MAXDISTANCE 2

/* Whether the battery model should be used */
#define USEBATTERY   1

/* The starting battery level */
#define BATTERYSTART 10000	

/* ID of the node that acts as the sink */
#define SINKNODE 0

/* Channel number for the messages */
#define MESSAGE_CHANNEL 7

/* maximum number of neighbours per node */
/* this value is not the whole neighborhood which is defined in */
/* the G constant */
#define MAXNEIGHBOR 10

/* define maximum number of children (for the tree) */
#define G 50

/* define the maximum size of the the packet treeVector */
#define MAX_SIZE 10

/* maximum size for chosenSenders and convincedSenser lists */
#define MAX_LIST 20

/* define the maximum size of the treeMatrix that contains de whole tree */
#define MAX_TREE 50 

/* special character */
/* uint8_t the limit value is 255 (1Byte=8bits) */ 
/* if we change the type we have to change the SPECIAL_CHAR */
//#define SPECIAL_CHAR 255
uint16_t SPECIAL_CHAR = 65535;

/* number of attemps to send a message */ 
#define ACK_ATTEMPTS 10

/* takes the K smallest numbers */
#define K 2

/* define the distance domination or depth */
#define R 2

/* define the distance domination or depth */
#define TIME_T 10

/* define the state of the node */
#define SLAVE 0
#define HEAD 1
#define ESCAPING 2

#define SLEEPING 0
#define FLOODINIT 1
#define FLOODING 2
#define HOPING 3


/* The datastructure for the messages 
 * For additions use the nx_ types.
 * E.g. if you want a int32_t use nx_int32_t
 */
 
typedef nx_struct nodeChangeType {
	nx_uint16_t nodeId;
	nx_uint16_t TTLState;
} nodeChange;

typedef nx_struct treeNeighborType {
	nx_uint16_t nodeId;  //the 2 last bits will encode the TTL
	nx_uint16_t father;  //the 2 last bits will encode the TTL
} treeVectorNeighbor;

typedef nx_union typeMsg{
	nx_uint16_t treeVector[MAX_SIZE];  //more size vector will crash 
	nx_uint16_t neighbourList[MAX_SIZE];
	nx_uint16_t stateUpdated[MAX_SIZE];	
	nx_uint16_t TTLVector[MAX_SIZE];
	nx_uint16_t joinVector[MAX_SIZE];
	nodeChange nodeChangeType[MAX_SIZE/2];
	treeVectorNeighbor treeNeighborType[MAX_SIZE/2];
}type_t;
 
//total length of the package payload is 28 Bytes
typedef nx_struct rout_msg {
  nx_uint8_t type; // [1Byte]
  nx_uint16_t from; // [2Byte]
  nx_uint8_t seq; // [1Byte] for the fragmented packet this field is going to be used as a fragment number
  nx_uint8_t content; //[1Byte] for the fragmented packet this field is going to be used as a fragment number
  type_t msgType; // [X Bytes]
} rout_msg_t;

#endif
