#ifndef ROUT_H
#define ROUT_H

/* Timer perdiod in milliseconds */
#define PERIOD 1000

/* Number of columns in topography */
#define COLUMNS 4

/* Number of different rounds in protocol and the rounds themselves */
#define ROUNDS 2
#define ROUND_ANNOUNCEMENT 0
#define ROUND_CONTENT 1

/* Message type identifiers */
#define TYPE_ANNOUNCEMENT 11
#define TYPE_CONTENT 12
#define TYPE_LIST 13
#define TYPE_LA 14
#define TYPE_NA 15
#define TYPE_NOTIFICATION_MSG 16
#define TYPE_JOIN 17
#define TYPE_NEIGHBOR_LIST 18

/* Maximum quadratic distance radio can reach */
//#define MAXDISTANCE 5
#define MAXDISTANCE 2

/* Whether the battery model should be used */
#define USEBATTERY   1

/* The starting battery level */
#define BATTERYSTART 10000	

/* Whether basic routing should be used */
#define BASICROUTER   0

/* ID of the node that acts as the sink */
#define SINKNODE 0

/* Channel number for the messages */
#define MESSAGE_CHANNEL 7

/* An approach of total nodes */
#define MAX_NODES 50

/* maximum number of neighbours per node */
#define MAXNEIGHBOR 6

/* special character */
/* because of the neighborMatrix and localMaxCliqueMatrix is*/
/* uint8_t the limit value is 255 (1Byte=8bits) */ 
#define SPECIAL_CHAR 255

/* number of attemps to send a message */ 
#define ACK_ATTEMPTS 10

/* defines the number of multiple domination */
#define K 3

/*define the distance domination */
#define R 4

/*define the status of the node */
#define DOMINATING 1 
#define PENDING_DOM 2
#define DOMINATED 3
#define GATEWAY 4


/* The datastructure for the messages 
 * For additions use the nx_ types.
 * E.g. if you want a int32_t use nx_int32_t
 *
 * Here you can add additional fields that you might need.
 */
 
typedef nx_union typeMsg{
	nx_uint8_t list[2][K];
	nx_uint8_t NA[MAXNEIGHBOR];
	nx_uint8_t neighbourList[MAXNEIGHBOR];	
}type_t;
 
typedef nx_struct rout_msg {
  nx_uint8_t  type;
  nx_uint16_t from;
  nx_uint32_t seq;
  nx_uint16_t content;
  type_t msgType;
} rout_msg_t;

#endif
