package main;
import java.util.*;

/* 
 * At the moment no new Grap objects are being created when
 * adding.  This might break stuff further on. We'll see how to deal
 * with it later on.
 *
 * The clone will not clone the contents in the tree.
 */

/**
 * Note that if you are walking with one GraphWalker, it is not safe
 * to invoke another one. For instance, GraphNode.toString() while
 * inside a walker helper will screw things up.
 */
class GraphNode<E> {
    
    private static int nextNodeId = 0;
    private static int getNextNodeId() {
	return nextNodeId++;
    }
    protected int nodeId;
    public int getNodeId() {
	return nodeId;
    }
    
    protected E content;

    protected LinkedList<GraphNode<E>> parents;
    protected LinkedList<GraphNode<E>> children;

    protected  GraphNode<E> tmpclone;
    public WalkerHelper wh = new WalkerHelper();

//     /**
//      * Copies all fields except for parent
//      */
//     public void copyFields(GraphNode<E> node) {
// 	content  = node.content;
// 	children = node.children;
//     }

    public GraphNode(E content) {
	nodeId = getNextNodeId();
	
	this.content = content;
	parents  = new LinkedList<GraphNode<E>>();
	children = new LinkedList<GraphNode<E>>();
    }

    public final List<GraphNode<E>> getChildren() {
	return children;
    }

    public final List<GraphNode<E>> getParents() {
	return parents;
    }

    public final E getContent() {
	return content;
    }

    public void setContent(E c) {
	content = c;
    }

    /* ============================================================ */


    private static <E> void addToListSet(List<GraphNode<E>> list, GraphNode<E> node) {
	for(GraphNode<E> n : list) {
	    if(n.getNodeId() == node.getNodeId()) {
		return;
	    }
	}
	list.add(node);
    }

    private static <E> int theIndexOf(List<GraphNode<E>> list, GraphNode<E> node) {
	boolean found = false;
	int index = -1;
	for(int i=0; i < list.size(); i++) {
	    GraphNode<E> n = list.get(i);
	    if(n.getNodeId() == node.getNodeId()) {
		if(!found) {
		    index = i;
		    found = true;
		} else {
		    throw new RuntimeException("List contains duplicate node.");
		}
	    }
	}
	if(!found) {
	    throw new RuntimeException("List does not contain node.");
	}
	return index;
    }

    private static <E> void replaceInListSet(List<GraphNode<E>> list, GraphNode<E> from, GraphNode<E> to) {
	int index = theIndexOf(list, from);
	list.set(index, to);
    }

    private static <E> void removeFromListSet(List<GraphNode<E>> list, GraphNode<E> node) {
	int index = theIndexOf(list, node);
	list.remove(index);
    }


    public void addChild(GraphNode<E> child) {
	addToListSet(children,      child);
	addToListSet(child.parents, this);
    }

    public void removeChild(GraphNode<E> child) {
	removeFromListSet(this.children, child);
	removeFromListSet(child.parents, this);
    }

    public void replaceMe(GraphNode<E> other) {
	for(GraphNode<E> p : parents) {
	    replaceInListSet(p.children, this, other);
	}
	parents = new LinkedList<GraphNode<E>>();
    }

    public void replaceChild(GraphNode<E> oldchild, GraphNode<E> newchild) {
	replaceInListSet(children, oldchild, newchild);
	removeFromListSet(oldchild.parents, this);
	addToListSet(     newchild.parents, this);
    }



//     /**
//      * Sets a child to a new tree. 
//      */
//     public void replaceChild(GraphNode<E> node) {
// 	boolean set = false;
// 	for(GraphNode<E> c : children) {
// 	    if(contentEquals(node)) {
// 		if(!set) {
// 		    c.copyFields(node);
// 		    set = true;
// 		} else {
// 		    throw new RuntimeException("Duplicate child!");
// 		}
// 	    }
// 	}
// 	if(!set) {
// 	    addChild(node);
// 	}
//     }

    public boolean haveChildWith(E fcontent) {
	return null != findChildWith(fcontent);
    }

    public GraphNode<E> findChildWith(E fcontent) {
 	for(GraphNode<E> c : children) {
	    //System.out.println("Comparing "+fcontent+" with "+c.content);
 	    if(c.content.equals(fcontent)) {
 		return c;
 	    }
 	}
 	return null;
    }


    public void removeChildren(Collection<GraphNode<E>> deletelist) {
	for(GraphNode<E> d : deletelist) {
	    removeChild(d);
	}
    }

    public void removeChildrenByContent(Collection<E> blacklist) {
        ArrayList<GraphNode<E>> deletelist = new ArrayList<GraphNode<E>>();
        for(GraphNode<E> n : children) {
            if(blacklist.contains(n.getContent())) {
                deletelist.add(n);
            }
	}
        removeChildren(deletelist);
    }

    public void removeChildren(List<TreeNode<E>> deletelist) {
	for(GraphNode<E> d : deletelist) {
	    removeChild(d);
	}
    }

    public void removeChildren() {
	while(children.size() > 0) {
	    removeChild(children.get(0));
	}
    }
    

    protected interface NodeFactory<E> {
	public GraphNode<E> newNode(E content);
    }

    protected class GraphNodeFactory<E> implements NodeFactory<E> {
	public GraphNode<E> newNode(E content) {
	    return new GraphNode<E>(content);
	}
    }


    public GraphNode<E> clone(NodeFactory nodefactory) {
	GraphWalker gw = new GraphWalker() 
	    {
		public <E> void visitNode(GraphNode<E> parent, GraphNode<E> node, GraphWalkerNodeDetails details, GraphWalkerWalkDetails walkdetails) {
		    if(!details.revisit && details.nstate == GraphWalker.NodeState.START) {
			
			Object o = walkdetails.visitobject;
			if(o instanceof NodeFactory) {
			    @SuppressWarnings("unchecked") NodeFactory<E> nodefactory = (NodeFactory<E>)o;
			    
			    
			    node.tmpclone = nodefactory.newNode(node.content);
			    if(parent != null) {
				if(details.pstate == GraphWalker.ParentState.FORWARD) {
				    parent.tmpclone.addChild(node.tmpclone);
				} else if(details.pstate == GraphWalker.ParentState.BACKWARD) {
				    node.tmpclone.addChild(parent.tmpclone);
				}
			    }
			    // System.out.println(node.getNodeId()+" cloned to "+node.tmpclone.getNodeId());
			} else {
			    throw new RuntimeException("Got wrong kind of visitobject! "+o.getClass());
			}
		    }
		}
	    };
	GraphWalkerWalkDetails wd = new GraphWalkerWalkDetails(true,true,nodefactory);
	gw.walk(this,wd);
	return tmpclone;
    }

    public GraphNode<E> clone() {
	return clone(new GraphNodeFactory());
    }
    
    public int sizeNodes() {
	GraphSizeWalker gw = new GraphSizeWalker();
	GraphWalkerWalkDetails wd = new GraphWalkerWalkDetails(true,true,null);
	gw.walk(this,wd);
	return gw.nodes;
    }

    public int sizeEdges() {
	GraphSizeWalker gw = new GraphSizeWalker();
	GraphWalkerWalkDetails wd = new GraphWalkerWalkDetails(true,true,null);
	gw.walk(this,wd);
	return gw.edges;
    }

    public boolean contentEquals(GraphNode<E> that) {
	return this.content.equals(that.content);
    }

    public String contentToString() {
	return content.toString();
    }

    /* ============================================================ */

    public static <E> List<E> getContents(List<GraphNode<E>> nodes) {
	List<E> contents = new LinkedList<E>();
	for(GraphNode<E> n : nodes) {
	    contents.add(n.getContent());
	}
	return contents;
    }



    public String walkToString() {
	StringBuffer b = new StringBuffer();
	GraphWalker gw = new GraphNodeStringWalker(b);
	GraphWalkerWalkDetails wd = new GraphWalkerWalkDetails(true,true,null);
	gw.walk(this,wd);
	return b.toString();
    }

    public String toString() {
	return walkToString();
    }
    
    public boolean gotChildren() {
	return children.size() > 0;
    }

    public boolean gotParents() {
	return parents.size() > 0;
    }

    public class WalkerHelper {
	public GraphNode<E> tmpclone;

	/* ========== */
	public Set<Integer> usedForwardLinks;
	public Set<Integer> usedBackwardLinks;

	public void resetLinks() {
	    usedForwardLinks = new HashSet<Integer>();
	    usedBackwardLinks = new HashSet<Integer>();
	}

	public <E> boolean isUsedLink(GraphNode<E> node, GraphWalker.ParentState pstate) {
	    int id = node.getNodeId();
	    switch(pstate) {
	    case FORWARD:
		return usedForwardLinks.contains(id);
	    case BACKWARD:
		return usedBackwardLinks.contains(id);
	    default:
		throw new RuntimeException("Unknown ParentState");
	    }
	}

	public <E> void useLink(GraphNode<E> node, GraphWalker.ParentState pstate) {
	    int id = node.getNodeId();
	    switch(pstate) {
	    case FORWARD:
		usedForwardLinks.add(id);
		break;
	    case BACKWARD:
		usedBackwardLinks.add(id);
		break;
	    default:
		throw new RuntimeException("Unknown ParentState");
	    }
	}
    }

}