package main;
import java.util.*;
import java.awt.Color;

public abstract class HeadStatistics {

    /**
     *
     */
    protected int networksize;
    
    /**
     * The nodes that claim to be heads
     */
    protected TreeSet<Integer> headnodes = new TreeSet<Integer>();

    /**
     * The nodes that are picked to be heads. Map from picked node to
     * the set och nodes that picked that one.
     */
    protected Map<Integer,TreeSet<Integer>> picknodes;

    /**
     * The nodes a node picks as its heads. Map from node to
     * its picksed nodes.
     */
    protected Map<Integer,TreeSet<Integer>> nodepicks;
    

    public HeadStatistics() {
        headnodes = new TreeSet<Integer>();
        picknodes = new TreeMap<Integer,TreeSet<Integer>>();
        nodepicks = new TreeMap<Integer,TreeSet<Integer>>();
    }

    public TreeSet<Integer> getHeadnodes() {
        return headnodes;
    }

    public void addHeadnodes(Collection<Integer> set) {
        headnodes.addAll(set);
    }

    private static void addToIntToIntSetMap(Map<Integer,TreeSet<Integer>> map, int key, int value) {
        if(map.containsKey(key)) {
            TreeSet<Integer> valueset = map.get(key);
            valueset.add(value);
        } else {
            TreeSet<Integer> valueset = new TreeSet<Integer>();
            valueset.add(value);
            map.put(key, valueset);
        }
    }

    public void addPickedNode(int ch, int picker) {
        addToIntToIntSetMap(picknodes, ch, picker);
        addToIntToIntSetMap(nodepicks, picker, ch);
    }

    public TreeSet<Integer> getPicknodes() {
        TreeSet<Integer> set = new TreeSet<Integer>();
        for(int id : picknodes.keySet()) {
            set.add(id);
        }
        return set;
    }

    public Map<Integer,TreeSet<Integer>> getPicknodesMap() {
        return picknodes;
    }

    public Map<Integer,TreeSet<Integer>> getNodepicksMap() {
        return picknodes;
    }


    public Collection<Integer> getNumberOfFollowersSet() {
        ArrayList<Integer> followers = new ArrayList<Integer>();
        for(TreeSet<Integer> set : picknodes.values()) {
            followers.add(set.size());
        }
        return followers;
    }



    public String commonHeadStatisticsToString() {
        MyStringBuffer b = new MyStringBuffer();
        Collection<Integer> followers = getNumberOfFollowersSet();
	b.appendln("Number of nodes: "+networksize);
	b.appendln("Ratio of claimed clusterhead nodes: "+(((double)headnodes.size()) / (double)networksize));
	b.appendln("Ratio of picked clusterhead nodes: "+(((double)picknodes.size()) / (double)networksize));
	b.appendln(Statistics.statisticsToString("Clusterhead followers (for picked clusterheads)", followers));
        return b.toString();
    }

    public void commonCollectHeadStatistics(Network net, NetworkNode.Type type) {
        networksize = net.size();
    }
    
    
    public abstract void collectHeadStatistics(Network net, NetworkNode.Type type);

    public abstract String headStatisticsToString();


    public static final Color errorColor = new Color(1.0f, 0.3f, 0.3f);
    public static final Color chosenColor = new Color(0.1f, 0.8f, 0.1f);
    public static final Color convincedColor = new Color(0.8f, 1.0f, 0.8f);
    public static final Color aloneColor = new Color(0.6f, 0.6f, 1.0f);
    public static final Color dominatingColor = chosenColor; //new Color(0.3f, 0.3f, 1.0f)

    public abstract Map<Integer,Color> getHeadColorMap();

    
    private static String intToIntSetToString(Map<Integer,TreeSet<Integer>> map) {
        MyStringBuffer b = new MyStringBuffer();
        for(Map.Entry<Integer,TreeSet<Integer>> entry : map.entrySet()) {
            int key = entry.getKey();
            b.appendln("      "+key+"="+entry.getValue());
        }
        return b.toString();
    }

    public String toString() {
        return commonHeadStatisticsToString();
    }
    
    public String commonDetailedHeadStatisticsToString() {
        MyStringBuffer b = new MyStringBuffer();
        b.appendln("HeadStatistics:");
        b.appendln("   headnodes: "+getHeadnodes());
        b.appendln("   Picked nodes: "+getPicknodes());
        b.appendln("   Picked nodes - picked -> set of pickers:");
        b.append(intToIntSetToString(picknodes));
        b.appendln("   Node picks - picker -> set of picked nodes:");
        b.append(intToIntSetToString(nodepicks));
        return b.toString();
    }

}