package main;

import java.util.*;

public class NetworkBuilder {
    
    NetworkNodeFactory factory;
    boolean negative;

    Map<Integer,NetworkNode> nodemap;
    Map<Integer,Coordinate> coordmap;

    public NetworkBuilder(NetworkNodeFactory factory, boolean negative) {
        this.factory = factory;
        this.negative = negative;

        nodemap  = new TreeMap<Integer,NetworkNode>();
        coordmap = new TreeMap<Integer,Coordinate>();
    }

    public NetworkBuilder(NetworkNodeFactory factory) {
        this(factory, false);
    }
    
    public NetworkNode getcreateNode(int id) {
        NetworkNode node;
        if(nodemap.containsKey(id)) {
            node = nodemap.get(id);
        } else {
            node = factory.newNode(id);
            nodemap.put(id, node);
        }
        return node;
    }

    public void add(int id, int x, int y, int[] neighbors) {
        if(negative) {
            id = -id;
        }
        NetworkNode node = getcreateNode(id);
        coordmap.put(id, new Coordinate(x,y));
        
        for(int i = 0; i < neighbors.length; i++) {
            int nid = neighbors[i];
            if(negative) {
                nid = -nid;
            }
            NetworkNode nn = getcreateNode(nid);
            node.addNeighbor(nn);
        }
    }
    
    public Network getNetwork() {
        SortedMap<Coordinate, NetworkNode> netmap = new TreeMap<Coordinate, NetworkNode>();
        for(int id : nodemap.keySet()) {
            Coordinate c   = coordmap.get(id);
            NetworkNode node = nodemap.get(id);
            if(c == null || node == null) {
                throw new RuntimeException("Got null in netmap:coord "+c+" and node "+node);
            }
            if(netmap.containsKey(c)) {
                throw new RuntimeException("Multiple nodes at the same coordinate: "+c);
            }
            netmap.put(c,node);
        }
        Network net =  new Network(netmap);
        if(!net.hasOnlyDoubleLinks()) {
            throw new RuntimeException("Links are mising!");
        }
        return net;
    }



}