package main;
import java.util.*;
import java.awt.Color;

/**
 * Class for producing a graph file for neato to layout.
 * The positions does not work, but the result is ok anyway
 */
public class NetworkDigraph {

    public static final int xmultiplier = 40;
    public static final int ymultiplier = 30;
    
    private static String numberToName(int number) {
	return "n"+(number < 0 ? "m"+(-number) : number);
    }

    private static String nodeToName(NetworkNode node) {
	return numberToName(node.getNumber());
    }

    public static String buildDigraph(Network net) {
	return buildDigraph(net, "G",null);
    }

    public static String buildDigraph(Network net, Map<Integer,Color> colormap) {
	return buildDigraph(net,"G",colormap);
    }
    
    public static String buildDigraph(Network net,String graphname, Map<Integer,Color> colormap) {
	SortedMap<Coordinate,NetworkNode> nodes = net.getNodes();
	
	/* using the Coordinates as int tuples for edges */
	TreeSet<Coordinate> edgeset = new TreeSet<Coordinate>();
	    
	StringBuffer b = new StringBuffer();
	for(Coordinate c : nodes.keySet()) {
	    NetworkNode n = nodes.get(c);
	    int number = n.getNumber();
	    String name = nodeToName(n);
	    int xpos  = c.getX() * xmultiplier;
	    int ypos  = c.getY() * ymultiplier;
	    
	    //[label=A, pos="52,234", width="0.75", height="0.50"]
	    
	    String colorstring = Digraph.getColorString(colormap,number);
	    b.append("   "+name+" [label=\""+number+"\""+colorstring+", pos=\""+xpos+","+ypos+"\"];\n");
	    List<NetworkNode> neighbors = n.getNeighbors();
	    for(int i=0; i < neighbors.size(); i++) {
		NetworkNode nn = neighbors.get(i);
		int first, last;
		int nnumber = nn.getNumber();
		if(number < nnumber) {
		    first = number;
		    last  = nnumber;
		} else {
		    first = nnumber;
		    last  = number;
		}
		Coordinate ec = new Coordinate(first,last);
		if(edgeset.add(ec)) {
		    b.append("   "+numberToName(first)+" -- "+numberToName(last)+";\n");
		}
	    }
	}
	return "graph "+graphname+" {\n"+b.toString()+"}\n";
    }
    
}