package main;
import java.util.*;

public abstract class NetworkNode {

    public enum Type {PDA, PDF, PDSA, PDSF, DOMSET, SPOHN, TMP};

    Network network;
    int number;
    List<NetworkNode> neighbors;
    ArrayList<NetworkMessage> inbox;

    GraphNode<Integer> tmpgn;

    protected NetworkNode() {
	;
    }

    public NetworkNode(Network network, int number) {
	this.network = network;
	this.number  = number;
	neighbors = new ArrayList<NetworkNode>();
	inbox     = new ArrayList<NetworkMessage>();
    }

    public void setNetwork(Network net) {
	network = net;
    }

    /**
     * Adds a node that this node can send messages to (not vice versa
     * unless the correspoding add is called).
     */
    public void addNeighbor(NetworkNode n) {
	neighbors.add(n);
    }

    /* Take one simulation step */
    public abstract void stepNode();

    /* Receive a NetworkMessage */
    public abstract void receiveMessage(NetworkMessage m);
    
    public void broadcast(NetworkMessage mess) {
	network.broadcast(this, mess);
    }

    public void storeMessage(NetworkMessage mess) {
	inbox.add(mess);
    }

    public List<NetworkNode> getNeighbors() {
	return neighbors;
    }

    public ArrayList<NetworkMessage> getInbox() {
	return inbox;
    }
    
    public int getNumber() {
	return number;
    }
 
    public abstract NetworkNodeFactory getFactory();
 
    public abstract String toJavaCodeTestClassString(String classname, Network net);

    /**
     * May throw a UnsupportedOperationException
     */
    public abstract void setArbitraryState(Random rand);

    /**
     * May throw a UnsupportedOperationException
     */
    public abstract void setBadState(Random rand, int input);

}