package main;
import java.util.*;

/**
 * Runs a modified spohn07 algorithm as that pseudocode has so many
 * type errors and other strangeness
 */
public class SpohnNode extends NetworkNode {

    SpohnData data;
    ReportMaker rep;

    int step = 0;
    
    private SpohnNode() {
        ;
    }

    public SpohnNode(Network net,SpohnData data) {
        super(net, data.id);
        this.data = data;
    }

    private void dominatingClearD() {
        data.D = new TreeMap<Integer,SpohnTuple>();
        data.D.put(data.id, new SpohnTuple(data.id, data.id, 0));
    }

    private void phaseOneHandleTuple(int from, SpohnTuple mtup) {
        int node = mtup.getNode();
        int dist = mtup.getDist() + 1;
        SpohnTuple h = data.H.get(node);
        rep.putLine(3,"Got tuple "+mtup+" from "+from);
        if(h == null || dist < h.getDist()) {
            rep.putLine(3,"Added it");
            SpohnTuple newh = new SpohnTuple(node, from, dist);
            data.H.put(node, newh);
        }
    }

    private void phaseOneHandle() {
        for(SpohnMessage mess : data.M) {
            int from = mess.getFrom();
            for(SpohnTuple mtup : mess.D) {
                phaseOneHandleTuple(from, mtup);
            }
        }
        
        data.D = new TreeMap<Integer, SpohnTuple>();
        for(SpohnTuple tup : data.H.values()) {
            data.D.put(tup.getNode(),tup);
            if(data.D.size() >= data.k) {
                break;
            }
        }
        rep.putLine(3,"H now contains "+data.H.values());
        rep.putLine(3,"D now contains "+data.D.values());
    }

    private SpohnMessage phaseOneMessage() {
        List<SpohnTuple> mtups = new ArrayList<SpohnTuple>();
        for(SpohnTuple tup : data.D.values()) {
            /* Only send an item if the neighbor can use it - not
             * appropriate for the D in phase two though */
            if(tup.getDist() < data.r) {
                mtups.add(tup);
            }
        }
        return new SpohnMessage(mtups, data.id);
    }

    private void phaseOneClose() {
        data.nextmessage = new SpohnMessage(SpohnMessage.Phase.TWO, data.id);
        if(data.D.containsKey(data.id)) {
            if(data.D.size() < data.k) {
                data.status = SpohnStatus.DOMINATING;
                addMessageNa(data.id);
            } else {
                data.status = SpohnStatus.PENDING;
            }
        } else {
            data.status = SpohnStatus.SLAVE;
        }
        rep.putLine(1,"Ends phase one as "+data.status);
    }

    private void phaseTwoOpen() {
        if(data.status == SpohnStatus.SLAVE) {
            for(SpohnTuple tup : data.D.values()) {
                data.nextmessage.D.add(tup);
                rep.putLine(3,"Will send out LocalAdvertisment("+tup+")");
            }
        }/* else {
            Added to try to fix the broken spohn algorithm: everyone schedules! */
        data.scheduledEvent = true;
        /* } */
    }


    private void addMessageNa(int id) {
        rep.putLine(3,"Will send out Na("+id+")");
        data.nextmessage.na.add(id, data.r);
    }

    private void addMessageNa(InfoFlood na) {
        rep.putLine(3,"Will send out Na: "+na);
        data.nextmessage.na.addInfoFlood(na);
    }

    private void addMessageNotification(int target,int hop) {
        SpohnUnicast su = new SpohnUnicast(target, hop);
        rep.putLine(3,"Will send out Notification("+su+")");
        data.nextmessage.notifications.add(su);
        data.notificationSent.add(target);
    }

    private void addMessageJoin(int target,int hop) {
        SpohnUnicast su = new SpohnUnicast(target, hop);
        rep.putLine(3,"Will send out Join("+su+")");
        data.nextmessage.joins.add(su);
        data.joinSent.add(target);
    }

    private boolean notificationSent(int target) {
        return data.notificationSent.contains(target);
    }

    private boolean joinSent(int target) {
        return data.joinSent.contains(target);
    }

    private int getNextHopTo(int target) {
        SpohnTuple tup = data.H.get(target);
        if(tup == null) {
            throw new RuntimeException("Cannot find a rout towards "+target+" at node "+data.id+" with routing table "+data.H);
        } else {
            return tup.getFrom();
        }
    }
    
    /* Shrinks D to k entries - never used now when everyone schedules events */
    private void shrinkD() {
        if(data.D.size() > data.k) {
            if(data.scheduledEvent) {
                rep.putLine(3,"Node "+data.id+" has a scheduled event, so no shrink of: "+data.D);
            } else {
                rep.putLine(3,"Got too big D: "+data.D);
                TreeSet<SpohnTuple> set = new TreeSet<SpohnTuple>();
                for(SpohnTuple tmp : data.D.values()) {
                    set.add(tmp);
                }
                data.D = new TreeMap<Integer,SpohnTuple>();
                for(int i= 0; i < data.k; i++) {
                    SpohnTuple tmp = set.pollFirst();
                    data.D.put(tmp.getNode(), tmp);
                }
                rep.putLine(3,"Removed entries "+set);
            }
        }
    }

    private void phaseTwoHandle() {
        data.nextmessage = new SpohnMessage(SpohnMessage.Phase.TWO, data.id);
        for(SpohnMessage mess : data.M) {
            rep.putLine(3,"--- Message from "+mess.getFrom()+" ---");
            int from = mess.getFrom();
            
            /* "case LocalAdvertisment" */
            for(SpohnTuple mtup : mess.D) {
                rep.putLine(2,"LocalAdvertisment: "+mtup);
                int n = mtup.getNode();
                int f = mtup.getFrom();
                int d = mtup.getDist();
                if(n == data.id && data.status != SpohnStatus.DOMINATING && data.status != SpohnStatus.PENDING) {
                    data.status = SpohnStatus.DOMINATING;
                    addMessageNa(data.id);
                    
                    /* Added: this was missing */
                    dominatingClearD();
                } else if(f == data.id) {
                    rep.putLine(2,"LocalAdvertisment: "+mtup);
                    if(data.status == SpohnStatus.SLAVE) {
                        data.status = SpohnStatus.GATEWAY;
                    }
                    if(!data.D.containsKey(n)) {
                        int hop = getNextHopTo(n);
                        if(hop != from) {
                            addMessageNotification(n,hop);
                        }
                    }
                }
            }
            
            /* "case Notification" */
            for(SpohnUnicast uc : mess.notifications) {
                if(uc.nexthop != data.id) {
                    /* Only deal with unicasts directoed to us */
                    continue;
                }
                rep.putLine(2,"Notification :"+uc);
                if(uc.target == data.id) {
                    /* "I am being notified" */
                    if(data.status != SpohnStatus.DOMINATING) {
                        data.status = SpohnStatus.DOMINATING;
                        addMessageNa(data.id);
                        dominatingClearD();
                    }
                } else if(!data.D.containsKey(uc.target)) { 
                    int hop = getNextHopTo(uc.target);
                    if(hop != mess.getFrom()) {
                        if(data.status == SpohnStatus.SLAVE) {
                            data.status = SpohnStatus.GATEWAY;
                        }
                        if(!notificationSent(uc.target)) {
                            addMessageNotification(uc.target, hop);
                        }
                    }
                }

            }

            /* "case Join" */
            for(SpohnUnicast uc : mess.joins) {
                if(uc.nexthop != data.id) {
                    /* Only deal with unicasts directoed to us */
                    continue;
                }
                rep.putLine(2,"Join :"+uc);
                if(uc.target != data.id) {
                    if(!notificationSent(uc.target) && !joinSent(uc.target)) {
                        if(data.status == SpohnStatus.SLAVE) {
                            data.status = SpohnStatus.GATEWAY;
                        }
                        int hop = getNextHopTo(uc.target);
                        addMessageJoin(uc.target, hop);
                    }
                } else { // I am the target of the join
                    /* This clause is added to fix the broken Spohn Algorithm */
                    if(!data.scheduledEvent) { // Already ran the event, now that everyone schedules
                        data.status = SpohnStatus.DOMINATING;
                        dominatingClearD();
                    }
                }
                
            }

            /* "case NeighborhoodAdvertisment" */
            for(Map.Entry<Integer,Integer> entry : mess.na.content.entrySet()) {
                rep.putLine(2,"Got Neighborhood Advertisment: "+entry);
                int node = entry.getKey();
                int ttl  = entry.getValue();
                int dist = data.r - ttl + 1;

                /* Added: Possibly add to H for routing purposes */
                SpohnTuple htup = new SpohnTuple(node, mess.getFrom(), dist);
                phaseOneHandleTuple(mess.getFrom(), htup);

                if(data.status != SpohnStatus.DOMINATING) {
                    SpohnTuple st    = data.D.get(node);
                    if(st != null) {
                        st.setValidated(true);
                    }
                    if(data.scheduledEvent) {
                        /* Added: no additions to D after scheduled Event */
                        if(st == null || dist < st.getDist()) {
                            data.D.put(node, new SpohnTuple(node, mess.getFrom(), dist, true));
                        }
                        //shrinkD(); // No use to shrink anymore
                    }
                }
            }
            mess.na.stepTTL();
            if(mess.na.size() > 0) {
                rep.putLine(3,"Relaying: "+data.H.values());
                addMessageNa(mess.na);
            }

            rep.putLine(3,"H now contains "+data.H.values());
            rep.putLine(3,"D now contains "+data.D.values());
        }
        
    }
    
    private void phaseTwoScheduledEvent() {
        data.scheduledEvent = false;
        rep.putLine(2,"Node "+data.id+" runs scheduled Event");
        TreeSet<SpohnTuple> validated = new TreeSet<SpohnTuple>();
        TreeSet<SpohnTuple> unvalidated = new TreeSet<SpohnTuple>();
        for(SpohnTuple tup : data.D.values()) {
            if(tup.isValidated()) {
                validated.add(tup);
            } else {
                /* To get sorting on distance instead of node */
                unvalidated.add(new SpohnTuple(tup.getDist(), tup.getFrom(), tup.getNode()));
            }
        }
        if(data.status == SpohnStatus.PENDING) {
            if(validated.size() < data.k) {
                data.status = SpohnStatus.DOMINATING;
                /* Added: this was missing */
                dominatingClearD();
            } else {
                if(data.notificationSent.size() > 0) {
                    data.status = SpohnStatus.GATEWAY;
                } else {
                    data.status = SpohnStatus.SLAVE;
                }
            }
        }
        if(data.status != SpohnStatus.DOMINATING) {
            data.D = new TreeMap<Integer,SpohnTuple>();
            if(validated.size() < data.k) {
                TreeSet<SpohnTuple> chosen = new TreeSet<SpohnTuple>();
                
                while(unvalidated.size() > 0 && validated.size() + chosen.size() < data.k) {
                    SpohnTuple fliptup = unvalidated.pollFirst();
                    chosen.add(new SpohnTuple(fliptup.getDist(), fliptup.getFrom(), fliptup.getNode()));
                }
                for(SpohnTuple tup : chosen) {
                    tup.setValidated(true);
                    addMessageJoin(tup.getNode(), tup.getFrom());
                    data.D.put(tup.getNode(),tup);
                }
            }
            for(SpohnTuple tup : validated) {
                addMessageJoin(tup.getNode(), tup.getFrom());
                data.D.put(tup.getNode(),tup);
                if(data.D.size() >= data.k) {
                    /* Added to maximum pick k nodes - should maybe chose the closest one, but what the heck. */
                    break;
                }
            }
        }
        /* Added to make the algorithm work with the added joins sent */
        data.notificationSent = new TreeSet<Integer>();
    }
    
    private SpohnMessage phaseTwoMessage() {
        if(data.nextmessage == null) {
            throw new RuntimeException("Got null nextmessage!");
        }
        return data.nextmessage;
    }
    

    public SpohnMessage stepNodeReturnMessage() {
        rep = new ReportMaker();
	rep.putLine(0, "");
	rep.putLine(0,"========== Report for node "+data.id+" step "+step+" running SPOHN ==========");

        SpohnMessage mess = null;
        if(data.rounds < data.getPhaseOneTime()) {
            rep.putLine(1, "Phase one");
            phaseOneHandle();
            mess = phaseOneMessage();
        } else if(data.rounds == data.getPhaseOneTime()) {
            rep.putLine(1, "Between phases");
            phaseOneHandle();
            phaseOneClose();
            phaseTwoOpen();
            mess = phaseTwoMessage();
        } else {
            rep.putLine(1, "Phase two");
            phaseTwoHandle();
            if(data.scheduledEvent && data.rounds == data.getScheduledTime()) {
                phaseTwoScheduledEvent();
            }
            mess = phaseTwoMessage();
        }
        rep.putLine(1,"Status: "+data.status);
 
        /* Prepare for next round*/
        data.rounds++;
        data.resetM();

        data.stepreport = rep.toString(3);
        data.nodereport.append(data.stepreport);
        
        step++;
        
        return mess;
    }

    public void receiveMessage(NetworkMessage nm) {
        if(nm == null) {
            throw new RuntimeException("Got null message!");
        }
        if(!(nm instanceof SpohnMessage)) {
            throw new RuntimeException("Got strange kind of message!");
        } else {
            data.M.add((SpohnMessage)nm);
        }
        
    }
    

    public boolean hasConverged() {
        int end = data.getConvergeTime();
        return data.rounds >= end;
    }

    public SpohnData getData() {
        return data;
    }

    public void stepNode() {
        broadcast(stepNodeReturnMessage());
    }

    


    public NetworkNodeFactory getFactory() {
        return new SpohnNodeFactory(data.k, data.r);
    }
 
    public String toJavaCodeTestClassString(String classname, Network net) {
        throw new UnsupportedOperationException();
    }

    public void setArbitraryState(Random rand) {
        throw new UnsupportedOperationException();
    }

    public void setBadState(Random rand, int input) {
       throw new UnsupportedOperationException();
    }

    public String getStepReport() {
        return data.stepreport;
    }

}