package main;
import java.util.*;

public class Statistics {

    public static <E extends Number> double mean(Collection<E> collection) {
	if(collection.size() < 1) {
	    //throw new RuntimeException("No elements");
	    return Double.NaN;
	}
	double sum = 0.0;
	for(Number n : collection) {
	    sum += n.doubleValue();
	}
	return sum / collection.size();
    }
    
    public static <E extends Number> double max(Collection<E> collection) {
	if(collection.size() < 1) {
	    //throw new RuntimeException("No elements");
	    return Double.NaN;
	}
	double max = Double.NEGATIVE_INFINITY;
	for(Number n : collection) {
	    double v = n.doubleValue();
	    if(v > max) {
		max = v;
	    }
	}
	return max;
    }

    public static <E extends Number> double min(Collection<E> collection) {
	if(collection.size() < 1) {
	    //throw new RuntimeException("No elements");
	    return Double.NaN;
	}
	double min = Double.POSITIVE_INFINITY;
	for(Number n : collection) {
	    double v = n.doubleValue();
	    if(v < min) {
		min = v;
	    }
	}
	return min;
    }

    public static <E extends Number> double stddev(Collection<E> collection) {
	if(collection.size() <= 1) {
	    //throw new RuntimeException("Too few elements");
	    return Double.NaN;
	}
	double m = mean(collection);
	double sum = 0.0;
	for(Number n : collection) {
	    double v = n.doubleValue();
	    double diff = v - m;
	    sum += diff * diff;
	}
	return Math.sqrt(sum / (collection.size() - 1));
    }

    public static <E extends Number> String statisticsToString(String title, Collection<E> collection) {
	StringBuffer b = new StringBuffer();
	b.append("--- "+title+" ---\n");
	b.append("Mean:   "+mean(collection)+"\n");
	b.append("Stddev: "+stddev(collection)+"\n");
	b.append("Min:    "+min(collection)+"\n");
	b.append("Max:    "+max(collection)+"\n");
	return b.toString();
    }
    

}